/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Based on the source code of legacyTray.js from GNOME Shell 3.24.

  Copyright (c) 2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Clutter = imports.gi.Clutter;
const GObject = imports.gi.GObject;
const Meta = imports.gi.Meta;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const CtrlAltTab = imports.ui.ctrlAltTab;
const Layout = imports.ui.layout;
const Main = imports.ui.main;
const Overview = imports.ui.overview;
const OverviewControls = imports.ui.overviewControls;
const Tweener = imports.ui.tweener;

const BARRIER_THRESHOLD = 70;
const BARRIER_TIMEOUT = 1000;
const CONCEALED_WIDTH = 3;
const REVEAL_ANIMATION_TIME = 0.2;
const TEMP_REVEAL_TIME = 2;

const LegacyTray = new Lang.Class({
    Name: 'LegacyTray',

    _init: function() {
        this.actor = new St.Widget({ clip_to_allocation: true, layout_manager: new Clutter.BinLayout() });
        let constraint = new Layout.MonitorConstraint({ primary: true, work_area: true });
        this.actor.add_constraint(constraint);
        this._slideLayout = new OverviewControls.SlideLayout();
        this._slideLayout.translationX = 0;
        this._slideLayout.slideDirection = OverviewControls.SlideDirection.LEFT;
        this._slider = new St.Widget({ x_expand: true, y_expand: true, x_align: Clutter.ActorAlign.START, y_align: Clutter.ActorAlign.END, layout_manager: this._slideLayout });
        this.actor.add_actor(this._slider);
        this._sliderChangedId = this._slider.connect('notify::allocation', Lang.bind(this, this._syncBarrier));
        this._box = new St.BoxLayout({ style_class: 'dschaerf-legacy-tray' });
        this._slider.add_actor(this._box);
        this._concealHandle = new St.Button({ style_class: 'dschaerf-legacy-tray-handle', accessible_name: _("Hide tray"), can_focus: true });
        this._concealHandle.child = new St.Icon({ icon_name: 'go-previous-symbolic' });
        this._box.add_child(this._concealHandle);
        this._iconBox = new St.BoxLayout({ style_class: 'dschaerf-legacy-tray-icon-box' });
        this._box.add_actor(this._iconBox);
        this._revealHandle = new St.Button({ style_class: 'dschaerf-legacy-tray-handle' });
        this._revealHandle.child = new St.Icon({ icon_name: 'go-next-symbolic' });
        this._box.add_child(this._revealHandle);
        this._revealHandle.bind_property('visible', this._concealHandle, 'visible', GObject.BindingFlags.BIDIRECTIONAL | GObject.BindingFlags.INVERT_BOOLEAN);
        this._revealHandleChangedId = [
            this._revealHandle.connect('notify::visible', Lang.bind(this, this._sync)),
            this._revealHandle.connect('notify::hover', Lang.bind(this, this._sync)),
            this._revealHandle.connect('clicked', Lang.bind(this, function() {
                this._concealHandle.show();
            }))
        ];
        this._concealHandleChangedId = this._concealHandle.connect('clicked', Lang.bind(this, function() {
            this._revealHandle.show();
        }));
        this._horizontalBarrier = null;
        this._pressureBarrier = new Layout.PressureBarrier(BARRIER_THRESHOLD, BARRIER_TIMEOUT, Shell.ActionMode.NORMAL);
        this._pressureBarrierChangedId = this._pressureBarrier.connect('trigger', Lang.bind(this, function() {
            this._concealHandle.show();
        }));
        Main.layoutManager.addChrome(this.actor, { affectsInputRegion: false });
        Main.layoutManager.trackChrome(this._slider, { affectsInputRegion: true });
        Main.uiGroup.set_child_below_sibling(this.actor, Main.layoutManager.modalDialogGroup);
        Main.ctrlAltTabManager.addGroup(this.actor, _("Status Icons"), 'focus-legacy-systray-symbolic', { sortGroup: CtrlAltTab.SortGroup.BOTTOM });
        this._trayManager = new Shell.TrayManager();
        this._trayManagerChangedId = [
            this._trayManager.connect('tray-icon-added', Lang.bind(this, this._onTrayIconAdded)),
            this._trayManager.connect('tray-icon-removed', Lang.bind(this, this._onTrayIconRemoved))
        ];
        if (global.screen) {
            this._trayManager.manage_screen(global.screen, this.actor);
        } else {
            this._trayManager.manage_screen(Main.panel.actor);
        }
        this._overviewChangedId = [
            Main.overview.connect('showing', Lang.bind(this, function() {
                Tweener.removeTweens(this._slider);
                Tweener.addTween(this._slider, { opacity: 0, time: Overview.ANIMATION_TIME, transition: 'easeOutQuad' });
            })),
            Main.overview.connect('shown', Lang.bind(this, this._sync)),
            Main.overview.connect('hiding', Lang.bind(this, function() {
                this._sync();
                Tweener.removeTweens(this._slider);
                Tweener.addTween(this._slider, { opacity: 255, time: Overview.ANIMATION_TIME, transition: 'easeOutQuad' });
            }))
        ];
        this._layoutManagerChangedId = Main.layoutManager.connect('monitors-changed', Lang.bind(this, this._sync));
        if (global.screen) this._screenChangedId = global.screen.connect('in-fullscreen-changed', Lang.bind(this, this._sync));
        this._sessionModeChangedId = Main.sessionMode.connect('updated', Lang.bind(this, this._sync));
        this._sync();
    },

    _onTrayIconAdded: function(tm, icon) {
        let wmClass = icon.wm_class ? icon.wm_class.toLowerCase() : '';
        let button = new St.Button({ child: icon, style_class: 'dschaerf-legacy-tray-icon', button_mask: St.ButtonMask.ONE | St.ButtonMask.TWO | St.ButtonMask.THREE, can_focus: true, x_fill: true, y_fill: true });
        let app = Shell.WindowTracker.get_default().get_app_from_pid(icon.pid);
        if (!app) app = Shell.AppSystem.get_default().lookup_startup_wmclass(wmClass);
        if (!app) app = Shell.AppSystem.get_default().lookup_desktop_wmclass(wmClass);
        if (app) {
            button.accessible_name = app.get_name();
        } else {
            button.accessible_name = icon.title;
        }
        button.connect('clicked', function() {
            icon.click(Clutter.get_current_event());
        });
        button.connect('key-press-event', function() {
            icon.click(Clutter.get_current_event());
            return Clutter.EVENT_PROPAGATE;
        });
        button.connect('key-focus-in', Lang.bind(this, function() {
            this._concealHandle.show();
        }));
        this._iconBox.add_child(button);
        if (!this._concealHandle.visible) {
            this._concealHandle.show();
            if (this._concealHandleTimeoutId) Mainloop.source_remove(this._concealHandleTimeoutId);
            this._concealHandleTimeoutId = Mainloop.timeout_add_seconds(TEMP_REVEAL_TIME, Lang.bind(this, function() {
                this._concealHandle.hide();
                return false;
            }));
        }
    },

    _onTrayIconRemoved: function(tm, icon) {
        if (!this.actor.contains(icon)) return;
        icon.get_parent().destroy();
        this._sync();
    },

    _sync: function() {
        let hasIcons = this._iconBox.get_n_children() > 0;
        let inOverview = Main.overview.visible && !Main.overview.animationInProgress;
        let inFullscreen = Main.layoutManager.primaryMonitor.inFullscreen;
        this.actor.visible = hasIcons && !inOverview && !inFullscreen;
        if (!hasIcons) this._concealHandle.hide();
        let targetSlide;
        if (this._concealHandle.visible) {
            targetSlide = 1.0;
        } else if (!hasIcons) {
            targetSlide = 0.0;
        } else {
            let [, boxWidth] = this._box.get_preferred_width(-1);
            let [, handleWidth] = this._revealHandle.get_preferred_width(-1);
            if (this._revealHandle.hover) {
                targetSlide = handleWidth / boxWidth;
            } else {
                targetSlide = CONCEALED_WIDTH / boxWidth;
            }
        }
        if (this.actor.visible) {
            Tweener.addTween(this._slideLayout, { slideX: targetSlide, time: REVEAL_ANIMATION_TIME, transition: 'easeOutQuad' });
        } else {
            this._slideLayout.slideX = targetSlide;
            this._unsetBarrier();
        }
    },

    _syncBarrier: function() {
        let rtl = (this._slider.get_text_direction() === Clutter.TextDirection.RTL);
        let [x, y] = this._slider.get_transformed_position();
        let [w, h] = this._slider.get_transformed_size();
        let x1 = Math.round(x);
        if (rtl) x1 += Math.round(w);
        let x2 = x1;
        let y1 = Math.round(y);
        let y2 = y1 + Math.round(h);
        if (this._horizontalBarrier && this._horizontalBarrier.x1 === x1 && this._horizontalBarrier.y1 === y1 && this._horizontalBarrier.x2 === x2 && this._horizontalBarrier.y2 === y2) return;
        this._unsetBarrier();
        let directions = (rtl ? Meta.BarrierDirection.NEGATIVE_X : Meta.BarrierDirection.POSITIVE_X);
        this._horizontalBarrier = new Meta.Barrier({ display: global.display, x1: x1, x2: x2, y1: y1, y2: y2, directions: directions });
        this._pressureBarrier.addBarrier(this._horizontalBarrier);
    },

    _unsetBarrier: function() {
        if (this._horizontalBarrier === null) return;
        this._pressureBarrier.removeBarrier(this._horizontalBarrier);
        this._horizontalBarrier.destroy();
        this._horizontalBarrier = null;
    },

    destroy: function() {
        if (this._concealHandleTimeoutId) Mainloop.source_remove(this._concealHandleTimeoutId);
        Main.sessionMode.disconnect(this._sessionModeChangedId);
        if (global.screen) global.screen.disconnect(this._screenChangedId);
        Main.layoutManager.disconnect(this._layoutManagerChangedId);
        this._overviewChangedId.forEach(function(id) {
            Main.overview.disconnect(id);
        }, this);
        this._trayManagerChangedId.forEach(function(id) {
            this._trayManager.disconnect(id);
        }, this);
        this._pressureBarrier.disconnect(this._pressureBarrierChangedId);
        this._concealHandle.disconnect(this._concealHandleChangedId);
        this._revealHandleChangedId.forEach(function(id) {
            this._revealHandle.disconnect(id);
        }, this);
        this._slider.disconnect(this._sliderChangedId);
        Tweener.removeTweens(this._slideLayout);
        Tweener.removeTweens(this._slider);
        Main.ctrlAltTabManager.removeGroup(this.actor);
        Main.uiGroup.remove_actor(this.actor);
        Main.layoutManager.untrackChrome(this._slider);
        Main.layoutManager.removeChrome(this.actor);
        this._unsetBarrier();
        this._pressureBarrier.destroy();
        this._revealHandle.destroy();
        this._iconBox.destroy();
        this._concealHandle.destroy();
        this._box.destroy();
        this._slider.destroy();
        this.actor.destroy();
    }
});

function init(metadata) {}

let _extension = null;

function enable() {
    if (_extension === null) _extension = new LegacyTray();
}

function disable() {
    if (_extension !== null) {
        _extension.destroy();
        _extension = null;
    }
}
