/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Copyright (c) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;

const Clutter = imports.gi.Clutter;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const _ = Convenience.getTranslations();

const FAVORITES_ALL = 1;
const FAVORITES_ICON = 1;
const FAVORITES_TEXT = 2;

const KEY_FAVORITE_APPS = 'favorite-apps';
const KEY_FAVORITES_SCOPE = 'favorites-scope';
const KEY_SHOW_FAVORITES = 'show-favorites';

const PopupFavoriteMenuItem = new Lang.Class({
    Name: 'PopupFavoriteMenuItem',
    Extends: PopupMenu.PopupBaseMenuItem,

    _init: function(app, params) {
        this._app = app;
        this.parent(params);
        this._icon = new St.Icon({ style_class: 'popup-menu-icon' });
        this._icon.gicon = this._app.get_app_info().get_icon();
        this.actor.add_child(this._icon);
        this._label = new St.Label({ text: this._app.get_name(), x_expand: true, y_align: Clutter.ActorAlign.CENTER });
        this.actor.add_child(this._label);
        this.actor.label_actor = this._label;
    },

    activate: function(event) {
        let modifiers = event ? event.get_state() : 0;
        let isCtrlPressed = (modifiers & Clutter.ModifierType.CONTROL_MASK) !== 0;
        let openNewWindow = this._app.can_open_new_window() && this._app.state === Shell.AppState.RUNNING && (isCtrlPressed);
        if (openNewWindow) {
            this._app.open_new_window(-1);
        } else {
            this._app.activate();
        }
        Main.overview.hide();
        this.parent(event);
    }
});

const FavoritesMenuButton = new Lang.Class({
    Name: 'FavoritesMenuButton',
    Extends: PanelMenu.Button,

    _init: function() {
        this.parent(1.0, _("Favorites"), false);
        this.menu.actor.add_style_class_name('dschaerf-menu');
        this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
        this._icon = new St.Icon({ icon_name: 'emblem-favorite-symbolic', style_class: 'system-status-icon' });
        this._hbox.add_child(this._icon);
        this._label = new St.Label({ text: _("Favorites"), y_align: Clutter.ActorAlign.CENTER });
        this._hbox.add_child(this._label);
        this._hbox.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
        this.actor.add_child(this._hbox);
        this._settingsChangedId = [
            Convenience.Settings.connect('changed::'+KEY_FAVORITE_APPS, Lang.bind(this, this._updateFavorites)),
            Convenience.Settings.connect('changed::'+KEY_FAVORITES_SCOPE, Lang.bind(this, this._updateFavorites)),
            Convenience.Settings.connect('changed::'+KEY_SHOW_FAVORITES, Lang.bind(this, this._onChangedShowFavorites))
        ];
        this._onChangedShowFavorites();
        this._appSystem = Shell.AppSystem.get_default();
        this._appSystemChangedId = this._appSystem.connect('installed-changed', Lang.bind(this, this._updateFavorites));
        this._globalSettingsChangedId = global.settings.connect('changed::'+KEY_FAVORITE_APPS, Lang.bind(this, this._updateFavorites));
        this._updateFavorites();
    },

    _onChangedShowFavorites: function() {
        let showFavorites = Convenience.Settings.get_enum(KEY_SHOW_FAVORITES);
        this._icon.visible = showFavorites !== FAVORITES_TEXT;
        this._label.visible = showFavorites !== FAVORITES_ICON;
    },

    _updateFavorites: function() {
        this.menu.removeAll();
        let appList = [];
        let favorites = Convenience.Settings.get_strv(KEY_FAVORITE_APPS)
        favorites.forEach(function(desktopFile) {
            let app = this._appSystem.lookup_app(desktopFile);
            if (app) appList.push(app);
        }, this);
        if (Convenience.Settings.get_enum(KEY_FAVORITES_SCOPE) === FAVORITES_ALL) {
            let favoriteApps = global.settings.get_strv(KEY_FAVORITE_APPS);
            favoriteApps.forEach(function(desktopFile) {
                let app = this._appSystem.lookup_app(desktopFile);
                if (app && appList.indexOf(app) < 0) appList.push(app);
            }, this);
        }
        appList.sort(function(a, b) {
            return a.compare_by_name(b);
        });
        appList.forEach(function(app) {
            this.menu.addMenuItem(new PopupFavoriteMenuItem(app));
        }, this);
        this.actor.visible = this.menu.numMenuItems > 0;
    },

    destroy: function() {
        global.settings.disconnect(this._globalSettingsChangedId);
        this._appSystem.disconnect(this._appSystemChangedId);
        this._settingsChangedId.forEach(function(id) {
            Convenience.Settings.disconnect(id);
        }, this);
        this.parent();
    }
});

function get() {
    return Main.panel.addToStatusArea('dschaerfFavoritesMenu', new FavoritesMenuButton(), 1, 'left');
}
