/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Copyright (c) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;

const Clutter = imports.gi.Clutter;
const Meta = imports.gi.Meta;
const Shell = imports.gi.Shell;
const St = imports.gi.St;

const Layout = imports.ui.layout;
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;

const ExtensionUtils = imports.misc.extensionUtils;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const HOT_CORNER_DEFAULT = 0;
const HOT_CORNER_ALTERNATIVE = 1;

const KEY_HOTCORNER_SENSITIVITY = 'hotcorner-sensitivity';

const KEY_SHOW_HOTCORNER = 'show-hotcorner';
const KEY_SHOW_HOTCORNER_LEFT = 'show-hotcorner-left';
const KEY_SHOW_HOTCORNER_RIGHT = 'show-hotcorner-right';

const HotCorner = new Lang.Class({
    Name: 'HotCorner',
    Extends: Layout.HotCorner,

    _init: function(layoutManager, monitor, x, y, side, size) {
        this.side = side;
        this._entered = false;
        this._monitor = monitor;
        this._x = x;
        this._y = y;
        this._setupFallbackCornerIfNeeded(layoutManager, size);
        this._pressureBarrier = new Layout.PressureBarrier(Layout.HOT_CORNER_PRESSURE_THRESHOLD, Layout.HOT_CORNER_PRESSURE_TIMEOUT, Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW);
        this._pressureBarrier.connect('trigger', Lang.bind(this, this._toggleOverview));
        this._ripple1 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple2 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        this._ripple3 = new St.BoxLayout({ style_class: 'ripple-box', opacity: 0, visible: false });
        let style = this.side === 'right' ? 'rtl' : 'ltr';
        this._ripple1.add_style_pseudo_class(style);
        this._ripple2.add_style_pseudo_class(style);
        this._ripple3.add_style_pseudo_class(style);
        layoutManager.uiGroup.add_actor(this._ripple1);
        layoutManager.uiGroup.add_actor(this._ripple2);
        layoutManager.uiGroup.add_actor(this._ripple3);
    },

    _animRipple: function(ripple, delay, time, startScale, startOpacity, finalScale) {
        ripple._opacity = startOpacity;
        ripple.set_anchor_point_from_gravity(this.side === 'right' ? Clutter.Gravity.NORTH_EAST : Clutter.Gravity.NORTH_WEST);
        ripple.visible = true;
        ripple.opacity = 255 * Math.sqrt(startOpacity);
        ripple.scale_x = ripple.scale_y = startScale;
        ripple.x = this._x;
        ripple.y = this._y;
        Tweener.addTween(ripple, { _opacity: 0, scale_x: finalScale, scale_y: finalScale, delay: delay, time: time, transition: 'linear', onUpdate: function() {
            ripple.opacity = 255 * Math.sqrt(ripple._opacity);
        }, onComplete: function() {
            ripple.visible = false;
        }});
    },

    _setupFallbackCornerIfNeeded: function(layoutManager, size) {
        if (!global.display.supports_extended_barriers() || size > 0) {
            this.actor = new Clutter.Actor({ name: 'hot-corner-environs', x: this._x, y: this._y, width: 3 + size, height: 3 + size, reactive: true });
            this._corner = new Clutter.Actor({ name: 'hot-corner', width: 1 + size, height: 1 + size, opacity: 0, reactive: true });
            this._corner._delegate = this;
            this.actor.add_child(this._corner);
            layoutManager.addChrome(this.actor);
            this._corner.set_position(this.side === 'right' ? this.actor.width - this._corner.width : 0, 0);
            this.actor.set_anchor_point_from_gravity(this.side === 'right' ? Clutter.Gravity.NORTH_EAST : Clutter.Gravity.NORTH_WEST);
            this.actor.connect('leave-event', Lang.bind(this, this._onEnvironsLeft));
            this._corner.connect('enter-event', Lang.bind(this, this._onCornerEntered));
            this._corner.connect('leave-event', Lang.bind(this, this._onCornerLeft));
        }
    },

    setBarrierSize: function(size) {
        if (this._verticalBarrier) {
            this._pressureBarrier.removeBarrier(this._verticalBarrier);
            this._verticalBarrier.destroy();
            this._verticalBarrier = null;
        }
        if (this._horizontalBarrier) {
            this._pressureBarrier.removeBarrier(this._horizontalBarrier);
            this._horizontalBarrier.destroy();
            this._horizontalBarrier = null;
        }
        if (size > 0) {
            if (this.side === 'right') {
                this._verticalBarrier = new Meta.Barrier({ display: global.display, x1: this._x, x2: this._x, y1: this._y, y2: this._y + size, directions: Meta.BarrierDirection.NEGATIVE_X });
                this._horizontalBarrier = new Meta.Barrier({ display: global.display, x1: this._x - size, x2: this._x, y1: this._y, y2: this._y, directions: Meta.BarrierDirection.POSITIVE_Y });
            } else {
                this._verticalBarrier = new Meta.Barrier({ display: global.display, x1: this._x, x2: this._x, y1: this._y, y2: this._y + size, directions: Meta.BarrierDirection.POSITIVE_X });
                this._horizontalBarrier = new Meta.Barrier({ display: global.display, x1: this._x, x2: this._x + size, y1: this._y, y2: this._y, directions: Meta.BarrierDirection.POSITIVE_Y });
            }
            this._pressureBarrier.addBarrier(this._verticalBarrier);
            this._pressureBarrier.addBarrier(this._horizontalBarrier);
        }
    }
});

const HotCorners = new Lang.Class({
    Name: 'HotCorners',

    _init: function() {
        this._hotCorners = []
        this._settingsChangedId = [
            Convenience.Settings.connect('changed::'+KEY_HOTCORNER_SENSITIVITY, Lang.bind(this, this._updateHotCorners)),
            Convenience.Settings.connect('changed::'+KEY_SHOW_HOTCORNER, Lang.bind(this, this._onChangedShowHotCorner)),
            Convenience.Settings.connect('changed::'+KEY_SHOW_HOTCORNER_LEFT, Lang.bind(this, this._onChangedShowHotCornerLeft)),
            Convenience.Settings.connect('changed::'+KEY_SHOW_HOTCORNER_RIGHT, Lang.bind(this, this._onChangedShowHotCornerRight))
        ];
        this._onChangedShowHotCorner();
        this._monitorManager = Meta.MonitorManager.get();
        this._monitorsChangedId = this._monitorManager.connect('monitors-changed', Lang.bind(this, this._updateHotCorners));
    },

    _changedLayoutManager: function(enabled) {
        if (enabled) {
            Main.layoutManager._updateHotCorners = function() {
                let showHotCorner = Convenience.Settings.get_enum(KEY_SHOW_HOTCORNER);
                if (showHotCorner === HOT_CORNER_DEFAULT) {
                    Layout.LayoutManager.prototype._updateHotCorners.call(this);
                } else {
                    this.hotCorners.forEach(function(corner) {
                        if (corner) corner.destroy();
                    });
                    this.hotCorners = [];
                    this.emit('hot-corners-changed');
                }
            };
        } else {
            Main.layoutManager._updateHotCorners = Layout.LayoutManager.prototype._updateHotCorners;
        }
        this._updateLayoutManager();
    },

    _clearHotCorners: function() {
        this._hotCorners.forEach(function(corner) {
            if (corner) corner.destroy();
        });
        this._hotCorners = [];
    },

    _createHotCorners: function(side) {
        let size = Main.layoutManager.panelBox.height;
        let sizeFallback = Convenience.Settings.get_int(KEY_HOTCORNER_SENSITIVITY) - 1;
        for (let i = 0; i < Main.layoutManager.monitors.length; i++) {
            let monitor = Main.layoutManager.monitors[i];
            let cornerX = side === 'right' ? monitor.x + monitor.width : monitor.x;
            let cornerY = monitor.y;
            let haveTopLeftCorner = true;
            if (i !== Main.layoutManager.primaryIndex) {
                let besideX = side === 'right' ? monitor.x + 1 : cornerX - 1;
                let besideY = cornerY;
                let aboveX = cornerX;
                let aboveY = cornerY - 1;
                for (let j = 0; j < Main.layoutManager.monitors.length; j++) {
                    if (i === j) continue;
                    let otherMonitor = Main.layoutManager.monitors[j];
                    if (besideX >= otherMonitor.x && besideX < otherMonitor.x + otherMonitor.width && besideY >= otherMonitor.y && besideY < otherMonitor.y + otherMonitor.height) {
                        haveTopLeftCorner = false;
                        break;
                    }
                    if (aboveX >= otherMonitor.x && aboveX < otherMonitor.x + otherMonitor.width && aboveY >= otherMonitor.y && aboveY < otherMonitor.y + otherMonitor.height) {
                        haveTopLeftCorner = false;
                        break;
                    }
                }
            }
            if (haveTopLeftCorner) {
                let corner = new HotCorner(Main.layoutManager, monitor, cornerX, cornerY, side, sizeFallback);
                corner.setBarrierSize(size);
                this._hotCorners.push(corner);
            } else {
                this._hotCorners.push(null);
            }
        }
    },

    _onChangedShowHotCorner: function() {
        this._changedLayoutManager(Convenience.Settings.get_enum(KEY_SHOW_HOTCORNER) !== HOT_CORNER_DEFAULT);
        this._updateHotCorners();
    },

    _onChangedShowHotCornerLeft: function() {
        let visible = Convenience.Settings.get_boolean(KEY_SHOW_HOTCORNER_LEFT);
        this._hotCorners.forEach(function(corner) {
            if (corner !== null && corner.side === 'left') {
                corner.setBarrierSize(visible ? Main.layoutManager.panelBox.height : 0);
                if (corner.actor) corner.actor.visible = visible;
            }
        });
    },

    _onChangedShowHotCornerRight: function() {
        let visible = Convenience.Settings.get_boolean(KEY_SHOW_HOTCORNER_RIGHT);
        this._hotCorners.forEach(function(corner) {
            if (corner !== null && corner.side === 'right') {
                corner.setBarrierSize(visible ? Main.layoutManager.panelBox.height : 0);
                if (corner.actor) corner.actor.visible = visible;
            }
        });
    },

    _updateHotCorners: function() {
        this._clearHotCorners();
        let showHotCorner = Convenience.Settings.get_enum(KEY_SHOW_HOTCORNER);
        switch (showHotCorner) {
        case HOT_CORNER_ALTERNATIVE:
            this._createHotCorners('left');
            this._createHotCorners('right');
            this._onChangedShowHotCornerLeft();
            this._onChangedShowHotCornerRight();
            break;
        case HOT_CORNER_DEFAULT:
            this._updateLayoutManager();
            break;
        default:
            // Nothing to do
        }
    },

    _updateLayoutManager: function() {
        Main.layoutManager._updateHotCorners.call(Main.layoutManager);
    },

    destroy: function() {
        this._monitorManager.disconnect(this._monitorsChangedId);
        this._settingsChangedId.forEach(function(id) {
            Convenience.Settings.disconnect(id);
        });
        this._clearHotCorners();
        this._changedLayoutManager(false);
    }
});

function get() {
    return new HotCorners();
}
