/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Copyright (c) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;

const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const St = imports.gi.St;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;
const PlacesManager = Me.imports.placesManager;
const UserDialog = Me.imports.userDialog;

const _ = Convenience.getTranslations();

const BOOKMARKS_MAINMENU = 0;

const KEY_BOOKMARKS_POSITION = 'bookmarks-position';
const KEY_SHOW_PLACES = 'show-places';

const PANELMENU_BUTTON_ICON = 1;
const PANELMENU_BUTTON_TEXT = 2;

const PopupPlaceMenuItem = new Lang.Class({
    Name: 'PopupPlaceMenuItem',
    Extends: PopupMenu.PopupBaseMenuItem,

    _init: function(text, iconName, params) {
        this.parent(params);
        this._icon = new St.Icon({ style_class: 'popup-menu-icon' });
        if (iconName) this._icon.icon_name = iconName;
        this.actor.add_child(this._icon);
        this._label = new St.Label({ text: text, x_expand: true, y_align: Clutter.ActorAlign.CENTER });
        this.actor.add_child(this._label);
        this.actor.label_actor = this._label;
    },

    addButton: function(params) {
        this._button = new St.Button({ style_class: 'dschaerf-button system-menu-action' });
        this._button.set_child(new St.Icon({ icon_name: params.iconName }));
        this._button.connect('clicked', Lang.bind(this, function(event) {
            let dialog = new UserDialog.ModalUserDialog(params.content, params.value);
            params.actions.forEach(function(action) {
                dialog.connect(action.signal, action.callback);
            }, this);
            dialog.open();
            Main.overview.hide();
            this.emit('activate', event);
        }, this));
        this.actor.add_child(this._button);
    },

    setIcon: function(icon) {
        this._icon.gicon = icon;
    }
});

const PopupPlaceInfoMenuItem = new Lang.Class({
    Name: 'PopupPlaceInfoMenuItem',
    Extends: PopupPlaceMenuItem,

    _init: function(info) {
        this.parent(info.name);
        this.setIcon(info.icon);
        if (info.canEject) this.addButton({
            iconName: 'media-eject-symbolic',
            content: 'EjectVolume',
            value: info.name,
            actions: [
                { signal: 'eject-volume', callback: Lang.bind(this, function() { info.eject(); })}
            ]
        });
        this._info = info;
    },

    activate: function(event) {
        this._info.launch(event.get_time());
        Main.overview.hide();
        this.parent(event);
    }
});

const PopupPlaceRecentMenuItem = new Lang.Class({
    Name: 'PopupPlaceRecentMenuItem',
    Extends: PopupPlaceMenuItem,

    _init: function(file, proxy) {
        this.parent(file.get_display_name());
        this.setIcon(Gio.content_type_get_icon(file.get_mime_type()));
        this.addButton({
            iconName: 'edit-clear-all-symbolic',
            content: 'RemoveRecentDocument',
            value: file.get_display_name(),
            actions: [
                { signal: 'remove-recent-document', callback: Lang.bind(this, function() { proxy.remove_item(file.get_uri()); }) }
            ]
        });
        this._file = file;
    },

    activate: function(event) {
        let launchContext = global.create_app_launch_context(event.get_time(), -1);
        Gio.AppInfo.launch_default_for_uri(this._file.get_uri(), launchContext);
        Main.overview.hide();
        this.parent(event);
    }
});

const PopupPlacesMenuSection = new Lang.Class({
    Name: 'PopupPlacesMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function(proxy, signal, callback) {
        this.parent();
        this._proxy = new PlacesManager[proxy]();
        this._proxyChangedId = this._proxy.connect(signal, Lang.bind(this, this[callback]));
        this[callback]();
    },

    _addPlacesCategory: function(menu, category) {
        let places = this._proxy.get(category);
        places.forEach(function(info) {
            menu.addMenuItem(new PopupPlaceInfoMenuItem(info));
        });
    },

    destroy: function() {
        this._proxy.disconnect(this._proxyChangedId);
        this._proxy.destroy();
        this.parent();
    }
});

const PopupPlacesBookmarksMenuSection = new Lang.Class({
    Name: 'PopupPlacesBookmarksMenuSection',
    Extends: PopupPlacesMenuSection,

    _init: function() {
        this.parent('PlacesBookmarksManager', 'updated::bookmarks', '_onUpdatedBookmarks');
        this._settingsChangedId = Convenience.Settings.connect('changed::'+KEY_BOOKMARKS_POSITION, Lang.bind(this, this._onChangedBookmarksPosition));
    },

    _onChangedBookmarksPosition: function() {
        this._onUpdatedBookmarks();
    },

    _onUpdatedBookmarks: function() {
        let item;
        this.removeAll();
        this._addPlacesCategory(this, 'bookmarks-special');
        if (Convenience.Settings.get_enum(KEY_BOOKMARKS_POSITION) === BOOKMARKS_MAINMENU) {
            this._addPlacesCategory(this, 'bookmarks-default');
        } else {
            item = new PopupMenu.PopupSubMenuMenuItem(_("Bookmarks"), true);
            item.icon.icon_name = 'user-bookmarks';
            this._addPlacesCategory(item.menu, 'bookmarks-default');
            if (item.menu.numMenuItems > 0) this.addMenuItem(item);
        }
        item = new PopupMenu.PopupSubMenuMenuItem(_("Bookmarks (custom)"), true);
        item.icon.icon_name = 'user-bookmarks';
        this._addPlacesCategory(item.menu, 'bookmarks-custom');
        if (item.menu.numMenuItems > 0) this.addMenuItem(item);
        this.actor.visible = this.numMenuItems > 0;
    },

    destroy: function() {
        Convenience.Settings.disconnect(this._settingsChangedId);
        this.parent();
    }
});

const PopupPlacesVolumesMenuSection = new Lang.Class({
    Name: 'PopupPlacesVolumesMenuSection',
    Extends: PopupPlacesMenuSection,

    _init: function() {
        this.parent('PlacesVolumesManager', 'updated::volumes', '_onUpdatedVolumes');
    },

    _onUpdatedVolumes: function() {
        let item;
        this.removeAll();
        this._addPlacesCategory(this, 'volumes-special');
        item = new PopupMenu.PopupSubMenuMenuItem(_("Network Places"), true);
        item.icon.icon_name = 'network-server';
        this._addPlacesCategory(item.menu, 'volumes-network');
        if (item.menu.numMenuItems > 0) this.addMenuItem(item);
        item = new PopupMenu.PopupSubMenuMenuItem(_("Removable Devices"), true);
        item.icon.icon_name = 'media-removable';
        this._addPlacesCategory(item.menu, 'volumes-device');
        if (item.menu.numMenuItems > 0) this.addMenuItem(item);
        this.actor.visible = this.numMenuItems > 0;
    }
});

const PopupPlacesRecentMenuSection = new Lang.Class({
    Name: 'PopupPlacesRecentMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function() {
        this.parent();
        this._itemClearRecent = new PopupPlaceMenuItem(_("Clear list of recent used documents…"), 'edit-clear-all');
        this._itemClearRecent.connect('activate', Lang.bind(this, function() {
            let dialog = new UserDialog.ModalUserDialog('ClearRecentDocuments');
            dialog.connect('clear-recent-documents', Lang.bind(this, function() {
                this._proxy.purge_items();
            }));
            dialog.connect('open-recent-documents', Lang.bind(this, function() {
                Gio.AppInfo.launch_default_for_uri('recent:///', global.create_app_launch_context(0, -1));
            }));
            dialog.open();
            Main.overview.hide();
        }));
        this.addMenuItem(this._itemClearRecent);
        this._itemDocuments = new PopupMenu.PopupSubMenuMenuItem(_("Recent used documents"), true);
        this._itemDocuments.icon.icon_name = 'document-open-recent';
        this.addMenuItem(this._itemDocuments);
        this._proxy = new Gtk.RecentManager();
        this._proxyChangedId = this._proxy.connect('changed', Lang.bind(this, this._onChangedRecent));
        this._onChangedRecent();
    },

    _onChangedRecent: function() {
        this._itemDocuments.menu.removeAll();
        if (this._proxy.size > 0) {
            let items = this._proxy.get_items();
            items.sort(function(a, b) {
                return GLib.utf8_collate(a.get_display_name(), b.get_display_name());
            });
            items.forEach(function(file) {
                if (GLib.file_test(file.get_uri_display(), GLib.FileTest.IS_REGULAR)) {
                    let item = new PopupPlaceRecentMenuItem(file, this._proxy);
                    this._itemDocuments.menu.addMenuItem(item);
                }
            }, this);
        }
        this.actor.visible = this._itemDocuments.menu.numMenuItems > 0;
    },

    destroy: function() {
        this._proxy.disconnect(this._proxyChangedId);
        this.parent();
    }
});

const PlacesMenuButton = new Lang.Class({
    Name: 'PlacesMenuButton',
    Extends: PanelMenu.Button,

    _init: function() {
        this.parent(1.0, _("Places"), false);
        this.menu.actor.add_style_class_name('dschaerf-menu');
        this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
        this._icon = new St.Icon({ icon_name: 'folder-symbolic', style_class: 'system-status-icon' });
        this._hbox.add_child(this._icon);
        this._label = new St.Label({ text: _("Places"), y_align: Clutter.ActorAlign.CENTER });
        this._hbox.add_child(this._label);
        this._hbox.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
        this.actor.add_actor(this._hbox);
        this._itemBookmarks = new PopupPlacesBookmarksMenuSection();
        this.menu.addMenuItem(this._itemBookmarks);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemVolumes = new PopupPlacesVolumesMenuSection();
        this.menu.addMenuItem(this._itemVolumes);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemRecent = new PopupPlacesRecentMenuSection();
        this.menu.addMenuItem(this._itemRecent);
        this._settingsChangedId = Convenience.Settings.connect('changed::'+KEY_SHOW_PLACES, Lang.bind(this, this._onChangedShowPlaces));
        this._onChangedShowPlaces();
    },

    _onChangedShowPlaces: function() {
        let showPlaces = Convenience.Settings.get_enum(KEY_SHOW_PLACES);
        this._icon.visible = showPlaces !== PANELMENU_BUTTON_TEXT;
        this._label.visible = showPlaces !== PANELMENU_BUTTON_ICON;
    },

    destroy: function() {
        Convenience.Settings.disconnect(this._settingsChangedId);
        this.parent();
    }
});

function get() {
    return Main.panel.addToStatusArea('PanelManagerPlacesMenuButton', new PlacesMenuButton(), 1, 'left');
}
