/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Copyright (c) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Signals = imports.signals;

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Shell = imports.gi.Shell;

const Main = imports.ui.main;
const ShellMountOperation = imports.ui.shellMountOperation;

const ExtensionUtils = imports.misc.extensionUtils;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const _ = Convenience.getTranslations();

const PlaceInfo = new Lang.Class({
    Name: 'PlaceInfo',

    _init: function(kind, file, name, iconName) {
        this.kind = kind;
        this.file = file;
        this.name = name || this._getFileName();
        this.icon = iconName ? new Gio.ThemedIcon({ name: iconName }) : this._getIcon();
    },

    _getFileName: function() {
        try {
            let info = this.file.query_info('standard::display-name', 0, null);
            return info.get_display_name();
        } catch (e) {
            if (e instanceof Gio.IOErrorEnum) return this.file.get_basename();
            throw e;
        }
    },

    _getIcon: function() {
        try {
            let info = this.file.query_info('standard::icon', 0, null);
            return info.get_icon();
        } catch (e) {
            if (e instanceof Gio.IOErrorEnum) {
                if (!this.file.is_native()) return new Gio.ThemedIcon({ name: 'folder-remote' });
                return new Gio.ThemedIcon({ name: 'folder' });
            } else {
                throw e;
            }
        }
    },

    launch: function(timestamp) {
        let launchContext = global.create_app_launch_context(timestamp, -1);
        try {
            Gio.AppInfo.launch_default_for_uri(this.file.get_uri(), launchContext);
        } catch (e) {
            if (e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NOT_MOUNTED)) {
                try {
                    this.file.mount_enclosing_volume(0, null, null, function(file, result) {
                        file.mount_enclosing_volume_finish(result);
                        Gio.AppInfo.launch_default_for_uri(file.get_uri(), launchContext);
                    });
                } catch (err) {
                    Main.notifyError(_("Failed to mount '%s'").format(this.name), err.message);
                }
            } else {
                Main.notifyError(_("Failed to launch '%s'").format(this.name), e.message);
            }
        }
    },

    get canEject() {
        return false;
    }
});

const PlaceMountInfo = new Lang.Class({
    Name: 'PlaceMountInfo',
    Extends: PlaceInfo,

    _init: function(kind, mount) {
        this._mount = mount;
        this.parent(kind, mount.get_root(), mount.get_name());
    },

    _ejectFinish: function(mount, result) {
        try {
            mount.eject_with_operation_finish(result);
        } catch (e) {
            Main.notifyError(_("Failed to eject '%s'").format(mount.get_name()), e.message);
        }
    },

    _getIcon: function() {
        return this._mount.get_icon();
    },

    _unmountFinish: function(mount, result) {
        try {
            mount.unmount_with_operation_finish(result);
        } catch (e) {
            Main.notifyError(_("Failed to unmount '%s'").format(mount.get_name()), e.message);
        }
    },

    eject: function() {
        let mountOp = new ShellMountOperation.ShellMountOperation(this._mount);
        if (this._mount.can_eject()) {
            this._mount.eject_with_operation(Gio.MountUnmountFlags.NONE, mountOp.mountOp, null, Lang.bind(this, this._ejectFinish));
        } else {
            this._mount.unmount_with_operation(Gio.MountUnmountFlags.NONE, mountOp.mountOp, null, Lang.bind(this, this._unmountFinish));
        }
    },

    get canEject() {
        if (!this._mount.can_eject() && !this._mount.can_unmount()) return false;
        let volume = this._mount.get_volume();
        return volume === null || volume.get_identifier('class') === 'device';
    }
});

const PlacesManager = new Lang.Class({
    Name: 'PlacesManager',

    _init: function() {
        this._places = {};
    },

    _addPlaceInfo: function(category, kind, file, name, iconName) {
        let info;
        try { info = new PlaceInfo(kind, file, name, iconName); } catch (e) { return; }
        this._places[category].push(info);
    },

    _sortCategory: function(category) {
        this._places[category].sort(function(a, b) {
            return GLib.utf8_collate(a.name, b.name);
        });
    },

    get: function(category) {
        return this._places[category];
    }
});
Signals.addSignalMethods(PlacesManager.prototype);

var PlacesBookmarksManager = new Lang.Class({
    Name: 'PlacesBookmarksManager',
    Extends: PlacesManager,

    _init: function() {
        this.parent();
        this._bookmarksFile = this._findBookmarksFile();
        this._bookmarksMonitor = null;
        this._bookmarksTimeoutId = 0;
        if (this._bookmarksFile) {
            this._bookmarksMonitor = this._bookmarksFile.monitor_file(Gio.FileMonitorFlags.NONE, null);
            this._bookmarksMonitorChangedId = this._bookmarksMonitor.connect('changed', Lang.bind(this, function() {
                if (this._bookmarksTimeoutId > 0) return;
                this._bookmarksTimeoutId = Mainloop.timeout_add(100, Lang.bind(this, function() {
                    this._bookmarksTimeoutId = 0;
                    this._updateBookmarks();
                    return false;
                }));
            }));
        }
        this._updateBookmarks();
    },

    _findBookmarksFile: function() {
        let paths = [
            GLib.build_filenamev([ GLib.get_user_config_dir(), 'gtk-3.0', 'bookmarks' ]),
            GLib.build_filenamev([ GLib.get_home_dir(), '.gtk-bookmarks' ])
        ];
        for (let i = 0; i < paths.length; i++) {
            if (GLib.file_test(paths[i], GLib.FileTest.EXISTS)) return Gio.File.new_for_path(paths[i]);
        }
        return null;
    },

    _updateBookmarks: function() {
        this._places['bookmarks-custom'] = [];
        this._places['bookmarks-default'] = [];
        this._places['bookmarks-special'] = [];
        this._addPlaceInfo('bookmarks-special', 'bookmark', Gio.File.new_for_path(GLib.get_home_dir()), _("Home Folder"));
        for (let i = 0; i < GLib.UserDirectory.N_DIRECTORIES; i++) {
            let specialPath = GLib.get_user_special_dir(i);
            if (!specialPath) continue;
            this._addPlaceInfo(i === GLib.UserDirectory.DIRECTORY_DESKTOP ? 'bookmarks-special' : 'bookmarks-default', 'bookmark', Gio.File.new_for_path(specialPath));
        }
        if (this._bookmarksFile) {
            let content = Shell.get_file_contents_utf8_sync(this._bookmarksFile.get_path());
            let lines = content.split('\n');
            for (let i = 0; i < lines.length; i++) {
                let line = lines[i];
                let components = line.split(' ');
                let bookmark = components[0];
                if (!bookmark) continue;
                let file = Gio.File.new_for_uri(bookmark);
                if (file.is_native() && !file.query_exists(null)) continue;
                let duplicate = false;
                for (let j = 0; j < this._places['bookmarks-special'].length; j++) {
                    if (file.equal(this._places['bookmarks-special'][j].file)) {
                        duplicate = true;
                        break;
                    }
                }
                if (duplicate) continue;
                for (let j = 0; j < this._places['bookmarks-default'].length; j++) {
                    if (file.equal(this._places['bookmarks-default'][j].file)) {
                        duplicate = true;
                        break;
                    }
                }
                if (duplicate) continue;
                this._addPlaceInfo('bookmarks-custom', 'bookmark', file, components[1] || null);
            }
        }
        this._sortCategory('bookmarks-default');
        this._sortCategory('bookmarks-custom');
        this.emit('updated::bookmarks');
    },

    destroy: function() {
        if (this._bookmarksTimeoutId > 0) Mainloop.source_remove(this._bookmarksTimeoutId);
        if (this._bookmarksMonitorChangedId) this._bookmarksMonitor.disconnect(this._bookmarksMonitorChangedId);
    }
});

var PlacesVolumesManager = new Lang.Class({
    Name: 'PlacesVolumesManager',
    Extends: PlacesManager,

    _init: function() {
        this.parent();
        this._volumeMonitor = Gio.VolumeMonitor.get();
        this._volumeMonitorChangedId = [
            this._volumeMonitor.connect('drive-changed', Lang.bind(this, this._updateVolumes)),
            this._volumeMonitor.connect('drive-connected', Lang.bind(this, this._updateVolumes)),
            this._volumeMonitor.connect('drive-disconnected', Lang.bind(this, this._updateVolumes)),
            this._volumeMonitor.connect('mount-added', Lang.bind(this, this._updateVolumes)),
            this._volumeMonitor.connect('mount-changed', Lang.bind(this, this._updateVolumes)),
            this._volumeMonitor.connect('mount-removed', Lang.bind(this, this._updateVolumes)),
            this._volumeMonitor.connect('volume-added', Lang.bind(this, this._updateVolumes)),
            this._volumeMonitor.connect('volume-changed', Lang.bind(this, this._updateVolumes)),
            this._volumeMonitor.connect('volume-removed',Lang.bind(this, this._updateVolumes))
        ];
        this._updateVolumes();
    },

    _addPlaceMountInfo: function(category, mount) {
        let info;
        try { info = new PlaceMountInfo(category, mount); } catch (e) { return; }
        this._places['volumes-'+category].push(info);
    },

    _updateVolumes: function() {
        this._places['volumes-device'] = [];
        this._places['volumes-network'] = [];
        this._places['volumes-special'] = [];
        this._addPlaceInfo('volumes-special', 'computer', Gio.File.new_for_uri('file:///'), _("Computer"), 'computer');
        this._addPlaceInfo('volumes-special', 'workgroup', Gio.File.new_for_uri('network:///'));
        let drives = this._volumeMonitor.get_connected_drives();
        drives.forEach(function(drive) {
            let volumes = drive.get_volumes();
            volumes.forEach(function(volume) {
                let mount = volume.get_mount();
                if (mount) {
                    let identifier = volume.get_identifier('class');
                    let category = identifier && identifier.indexOf('network') >= 0 ? 'network' : 'device';
                    this._addPlaceMountInfo(category, mount);
                }
            }, this);
        }, this);
        let volumes = this._volumeMonitor.get_volumes();
        volumes.forEach(function(volume) {
            if (volume.get_drive() === null) {
                let mount = volume.get_mount();
                if (mount) {
                    let identifier = volume.get_identifier('class');
                    let category = identifier && identifier.indexOf('network') >= 0 ? 'network' : 'device';
                    this._addPlaceMountInfo(category, mount);
                }
            }
        }, this);
        let mounts = this._volumeMonitor.get_mounts();
        mounts.forEach(function(mount) {
            if (!mount.is_shadowed() && !mount.get_volume()) {
                let location = mount.get_default_location();
                let category = location.is_native() ? 'device' : 'network';
                this._addPlaceMountInfo(category, mount);
            }
        }, this);
        this._sortCategory('volumes-device');
        this._sortCategory('volumes-network');
        this.emit('updated::volumes');
    },

    destroy: function() {
        this._volumeMonitorChangedId.forEach(function(id) {
            this._volumeMonitor.disconnect(id);
        }, this);
    }
});
