/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Copyright (c) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;

const Shell = imports.gi.Shell;
const St = imports.gi.St;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const _ = Convenience.getTranslations();

const BASIC_SETTINGS = [
    'ca.desrt.dconf-editor.desktop',        // dconf-editor
    'gconf-editor.desktop',                 // gconf-editor
    'gnome-control-center.desktop',         // gnome-control-center
    'gnome-session-properties.desktop',     // gnome-session-common (OLD -> Startup Applications)
    'gnome-shell-extension-prefs.desktop',  // gnome-shell
    'gnome-tweak-tool.desktop',             // gnome-tweak-tool
    'org.gnome.tweaks.desktop',             // gnome-tweaks
    'system-config-printer.desktop',        // system-config-printer
    'synaptic.desktop',                     // synaptic
    'users.desktop'                         // gnome-system-tools
]

const KEY_SHOW_AGGREGATE_MENU = 'show-systemmenu';

const AGGREGATE_MENU_DEFAULT = 0;
const AGGREGATE_MENU_ICON = 1;

let _aggregateMenu = Main.panel.statusArea.aggregateMenu;

const BasicSettingsIndicator = new Lang.Class({
    Name: 'BasicSettingsIndicator',
    Extends: PanelMenu.SystemIndicator,

    _init: function() {
        this.parent();
        this._item = new PopupMenu.PopupSubMenuMenuItem(_("Basic Settings"), true);
        this._item.icon.icon_name = 'preferences-system-symbolic';
        this.menu.addMenuItem(this._item);
        this._appSystem = Shell.AppSystem.get_default();
        this._appSystemChangedId = this._appSystem.connect('installed-changed', Lang.bind(this, this._updateApplications));
        this._updateApplications();
        this._menuChangedId = this.menu.connect('open-state-changed', Lang.bind(this, function(menu, isOpen) {
            if (isOpen) this._updateVisibility(!menu.actor.visible);
        }));
    },

    _updateApplications: function() {
        this._item.menu.removeAll();
        let appList = [];
        BASIC_SETTINGS.forEach(function(desktopFile) {
            let app = this._appSystem.lookup_app(desktopFile);
            if (app) appList.push(app);
        }, this);
        appList.sort(function(a, b) {
            return a.compare_by_name(b);
        });
        appList.forEach(function(app) {
            this._item.menu.addAction(app.get_name(), function() {
                app.activate();
                Main.overview.hide();
            });
        }, this);
        this.menu.actor.visible = this._item.menu.numMenuItems > 0;
    },

    _updateVisibility: function(visible) {
        _aggregateMenu._system._settingsAction.visible = visible;
    },

    destroy: function() {
        this._appSystem.disconnect(this._appSystemChangedId);
        this.menu.disconnect(this._menuChangedId);
        this.menu.destroy();
        this._updateVisibility(true);
    }
});

const AggregateMenu = new Lang.Class({
    Name: 'AggregateMenu',

    _init: function() {
        _aggregateMenu.actor.remove_actor(_aggregateMenu._indicators);
        this._indicators = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
        this._indicators.add_child(_aggregateMenu._indicators);
        let box = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
        box.add_child(new St.Icon({ icon_name: 'system-shutdown-symbolic', style_class: 'system-status-icon' }));
        box.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
        this._indicators.add_child(box);
        _aggregateMenu.actor.add_child(this._indicators);
        this._basicSettings = new BasicSettingsIndicator();
        _aggregateMenu.menu.addMenuItem(this._basicSettings.menu, _aggregateMenu.menu.numMenuItems - 1);
        this._settingsChangedId = Convenience.Settings.connect('changed::'+KEY_SHOW_AGGREGATE_MENU, Lang.bind(this, this._onChangedShowAggregateMenu));
        this._onChangedShowAggregateMenu();
    },

    _onChangedShowAggregateMenu: function() {
        let showAggregateMenu = Convenience.Settings.get_enum(KEY_SHOW_AGGREGATE_MENU);
        let children = this._indicators.get_children();
        children[0].visible = showAggregateMenu === AGGREGATE_MENU_DEFAULT;
        children[1].visible = showAggregateMenu === AGGREGATE_MENU_ICON;
    },

    destroy: function() {
        Convenience.Settings.disconnect(this._settingsChangedId);
        _aggregateMenu.actor.remove_actor(this._indicators);
        this._indicators.remove_actor(_aggregateMenu._indicators);
        _aggregateMenu.actor.add_child(_aggregateMenu._indicators);
        _aggregateMenu._indicators.show();
        this._basicSettings.destroy();
        this._indicators.destroy_all_children();
        this._indicators.destroy();
    }
});

function get() {
    return new AggregateMenu();
}
