/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Copyright (c) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;
const Mainloop = imports.mainloop;

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Meta = imports.gi.Meta;
const Shell = imports.gi.Shell;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;
const StatusSystem = imports.ui.status.system;

const ExtensionUtils = imports.misc.extensionUtils;
const LoginManager = imports.misc.loginManager;
const Util = imports.misc.util;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;
const UserDialog = Me.imports.userDialog;

const _ = Convenience.getTranslations();

const DISABLE_RESTART_BUTTONS = 'disable-restart-buttons';

const GNOME_TERMINAL = [
    'gnome-terminal.desktop',
    'org.gnome.Terminal.desktop'
];

let [ _system, _systemActions ] = (function() {
    let system = Main.panel.statusArea.aggregateMenu._system;
    return [ system, Convenience.checkVersion([ '3.18', '3.20', '3.22', '3.24' ]) ? system : system._systemActions ];
})();

const SystemIndicator = new Lang.Class({
    Name: 'SystemIndicator',

    _init: function() {
        this._actionSettings = _system._createActionButton('preferences-other-symbolic', _("Settings for PanelManager"));
        this._actionSettings.connect('clicked', Lang.bind(this, this._onClickedSettings));
        this._actionTerminal = _system._createActionButton('utilities-terminal-symbolic', _("Terminal"));
        this._actionTerminal.connect('clicked', Lang.bind(this, this._onClickedTerminal));
        this._actionRestart = _system._createActionButton('view-refresh-symbolic', _("GNOME Shell restart"));
        this._actionRestart.connect('clicked', Lang.bind(this, this._onClickedGNOMEShellRestart));
        this._actionHibernate = _system._createActionButton('document-save-symbolic', _("Hibernate"));
        this._actionHibernate.connect('clicked', Lang.bind(this, this._onClickedHibernate));
        this._actionHybridSleep = _system._createActionButton('document-save-as-symbolic', _("Hybrid-Sleep"));
        this._actionHybridSleep.connect('clicked', Lang.bind(this, this._onClickedHybridSleep));
        this._altSwitcher = new StatusSystem.AltSwitcher(this._actionHibernate, this._actionHybridSleep);
        let buttons = [];
        let children = _system._actionsItem.actor.get_children();
        for (let i = 1; i < children.length; i++) {
            _system._actionsItem.actor.remove_actor(children[i]);
            buttons.push(children[i]);
        }
        for (let i = 0; i < buttons.length; i++) {
            if (i === 1) {
                _system._actionsItem.actor.add(this._actionSettings, { expand: true, x_fill: false });
                _system._actionsItem.actor.add(this._actionTerminal, { expand: true, x_fill: false });
                _system._actionsItem.actor.add(this._actionRestart, { expand: true, x_fill: false });
            } else if (i === buttons.length - 1) {
                _system._actionsItem.actor.add(this._altSwitcher.actor, { expand: true, x_fill: false });
            }
            _system._actionsItem.actor.add(buttons[i], { expand: true, x_fill: false });
        }
        _systemActions._updateLockScreen.call(_systemActions);
        _systemActions._updateOrientationLock.call(_systemActions);
        _systemActions._updatePowerOff.call(_systemActions);
        _systemActions._updateSuspend.call(_systemActions);
        this._appSystem = Shell.AppSystem.get_default();
        this._appSystemChangedId = this._appSystem.connect('installed-changed', Lang.bind(this, this._updateApplications));
        this._updateApplications();
        this._loginManager = LoginManager.getLoginManager();
        this._openStateChangedId = _system.menu.connect('open-state-changed', Lang.bind(this, function(menu, isOpen) {
            if (!isOpen) return;
            this._actionRestart.visible = Meta.is_wayland_compositor() ? false : true;
            this._updateHibernate();
            this._updateHybridSleep();
        }));
    },

    _canShutdown: function(value, asyncCallback) {
        if (this._loginManager._proxy) {
            this._loginManager._proxy.call(value, null, Gio.DBusCallFlags.NONE, -1, null, function(proxy, asyncResult) {
                try {
                    let result = proxy.call_finish(asyncResult).deep_unpack();
                    let disableRestartButton = _systemActions._loginScreenSettings.get_boolean(DISABLE_RESTART_BUTTONS);
                    let disabled = Main.sessionMode.isLocked || (Main.sessionMode.isGreeter && disableRestartButton);
                    asyncCallback(result[0] !== 'no' && !disabled);
                } catch(e) {
                    asyncCallback(false);
                }
            });
        } else {
            Mainloop.idle_add(Lang.bind(this, function() {
                asyncCallback(false);
                return false;
            }));
        }
    },

    _onClickedGNOMEShellRestart: function(animate) {
        _system.menu.itemActivated(animate);
        Meta.restart(_("Restarting…"));
    },

    _onClickedHibernate: function(animate) {
        _system.menu.itemActivated(animate);
        if (this._actionHybridSleep.visible) {
            this._dialog = new UserDialog.ModalUserDialog('HibernateHybridSleep');
            this._dialog.connect('hybrid-sleep', Lang.bind(this, this._onClickedHybridSleep));
        } else {
            this._dialog = new UserDialog.ModalUserDialog('Hibernate');
        }
        this._dialog.connect('hibernate', Lang.bind(this, function() {
            if (this._loginManager._proxy) {
                this._loginManager._proxy.call("Hibernate", GLib.Variant.new('(b)', [true]), Gio.DBusCallFlags.NONE, -1, null, null);
            } else {
                this._loginManager.emit('prepare-for-sleep', true);
                this._loginManager.emit('prepare-for-sleep', false);
            }
        }));
        this._dialog.open();
        Main.overview.hide();
    },

    _onClickedHybridSleep: function(animate) {
        _system.menu.itemActivated(animate);
        this._dialog = new UserDialog.ModalUserDialog('HybridSleep');
        this._dialog.connect('hybrid-sleep', Lang.bind(this, function() {
            if (this._loginManager._proxy) {
                this._loginManager._proxy.call("HybridSleep", GLib.Variant.new('(b)', [true]), Gio.DBusCallFlags.NONE, -1, null, null);
            } else {
                this._loginManager.emit('prepare-for-sleep', true);
                this._loginManager.emit('prepare-for-sleep', false);
            }
        }));
        this._dialog.open();
        Main.overview.hide();
    },

    _onClickedSettings: function(animate) {
        _system.menu.itemActivated(animate);
        Util.spawnCommandLine('gnome-shell-extension-prefs '+Me.metadata['uuid']);
        Main.overview.hide();
    },

    _onClickedTerminal: function(animate) {
        _system.menu.itemActivated(animate);
        Util.spawnCommandLine('gnome-terminal');
        Main.overview.hide();
    },

    _updateApplications: function() {
        let gnomeTerminal = false;
        GNOME_TERMINAL.forEach(function(desktopFile) {
            let app = this._appSystem.lookup_app(desktopFile);
            if (app) gnomeTerminal = true;
        }, this);
        this._actionTerminal.visible = gnomeTerminal;
    },

    _updateHibernate: function() {
        this._canShutdown("CanHibernate", Lang.bind(this, function(result) {
            this._actionHibernate.visible = result;
        }));
    },

    _updateHybridSleep: function() {
        this._canShutdown("CanHybridSleep", Lang.bind(this, function(result) {
            this._actionHybridSleep.visible = result;
        }));
    },

    destroy: function() {
        this._appSystem.disconnect(this._appSystemChangedId);
        _system.menu.disconnect(this._openStateChangedId);
        _system._actionsItem.actor.remove_actor(this._altSwitcher.actor);
        _system._actionsItem.actor.remove_actor(this._actionRestart);
        _system._actionsItem.actor.remove_actor(this._actionTerminal);
        _system._actionsItem.actor.remove_actor(this._actionSettings);
        this._actionHybridSleep.destroy();
        this._actionHibernate.destroy();
        this._altSwitcher.actor.destroy();
        this._actionRestart.destroy();
        this._actionTerminal.destroy();
        this._actionSettings.destroy();
    }
});

function get() {
    return new SystemIndicator();
}
