/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Copyright (c) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;

const Gvc = imports.gi.Gvc;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;
const StatusVolume = imports.ui.status.volume;

let _volume = Main.panel.statusArea.aggregateMenu._volume;

const PopupVolumeSubMenuMenuItem = new Lang.Class({
    Name: 'PopupVolumeSubMenuMenuItem',
    Extends: PopupMenu.PopupSubMenuMenuItem,

    _init: function(deviceType) {
        this.parent('...', true);
        this._controlChangedId = [
            _volume._control.connect('state-changed', Lang.bind(this, this._updateDevices)),
            _volume._control.connect(deviceType+'-added', Lang.bind(this, this._updateDevices)),
            _volume._control.connect(deviceType+'-removed', Lang.bind(this, this._updateDevices)),
            _volume._control.connect('active-'+deviceType+'-update', Lang.bind(this, this._setActiveDevice))
        ];
        this._updateDevices();
    },

    _addDevice: function(id) {
        let uidevice = this.getDeviceById(id);
        if (!uidevice || !uidevice.port_available) return null;
        let deviceInfo = new Object();
        deviceInfo.uidevice = uidevice;
        deviceInfo.text = uidevice.description;
        if (uidevice.origin !== '') deviceInfo.text += ' - '+uidevice.origin;
        deviceInfo.item = this.menu.addAction(deviceInfo.text, Lang.bind(this, function() {
            this.changeDevice(uidevice);
        }));
        this._devices[id] = deviceInfo;
        return uidevice;
    },

    _setActiveDevice: function(control, id) {
        let deviceInfo = this._devices[id];
        if (deviceInfo && deviceInfo !== this._activeDevice) {
            if (this._activeDevice) this._activeDevice.item.setOrnament(PopupMenu.Ornament.NONE);
            this._activeDevice = deviceInfo;
            deviceInfo.item.setOrnament(PopupMenu.Ornament.CHECK);
            this.label.text = deviceInfo.text;
            let icon = deviceInfo.uidevice.get_icon_name();
            if (icon === null || icon.trim() === '') icon = this.getDefaultIcon();
            this.icon.icon_name = icon;
        }
    },

    _updateDevices: function() {
        this.menu.removeAll();
        this._devices = {};
        if (_volume._control.get_state() === Gvc.MixerControlState.READY) {
            let defaultStream = this.getDefaultStream();
            let numId = new Gvc.MixerUIDevice().get_id();
            for (let id = 0; id <= numId; id++) {
                let uidevice = this._addDevice(id);
                if (uidevice) {
                    let stream = _volume._control.get_stream_from_device(uidevice);
                    if (stream) {
                        let stream_port = stream.get_port();
                        if (stream_port && stream === defaultStream && stream_port.port === uidevice.get_port()) this._setActiveDevice(_volume._control, id);
                    }
                }
            }
            this.actor.visible = this.menu.numMenuItems > 0;
        }
    },

    destroy: function() {
        this._controlChangedId.forEach(function(id) {
            _volume._control.disconnect(id);
        }, this);
        this.parent();
    }
});

const PopupVolumeInputSubMenuMenuItem = new Lang.Class({
    Name: 'PopupVolumeInputSubMenuMenuItem',
    Extends: PopupVolumeSubMenuMenuItem,

    _init: function() {
        this.parent('input');
    },

    changeDevice: function(uidevice) {
        _volume._control.change_input(uidevice);
    },

    getDefaultStream: function() {
        return _volume._control.get_default_source();
    },

    getDefaultIcon: function() {
        return 'audio-input-microphone';
    },

    getDeviceById: function(id) {
        return _volume._control.lookup_input_id(id);
    }
});

const PopupVolumeOutputSubMenuMenuItem = new Lang.Class({
    Name: 'PopupVolumeOutputSubMenuMenuItem',
    Extends: PopupVolumeSubMenuMenuItem,

    _init: function() {
        this.parent('output');
    },

    changeDevice: function(uidevice) {
        _volume._control.change_output(uidevice);
    },

    getDefaultStream: function() {
        return _volume._control.get_default_sink();
    },

    getDefaultIcon: function() {
        return 'audio-card';
    },

    getDeviceById: function(id) {
        return _volume._control.lookup_output_id(id);
    }
});

const VolumeIndicator = new Lang.Class({
    Name: 'VolumeIndicator',

    _init: function() {
        this._secondaryIndicator = _volume._addIndicator();
        this._itemOutput = new PopupVolumeOutputSubMenuMenuItem();
        _volume._volumeMenu.addMenuItem(this._itemOutput, 1);
        this._itemInput = new PopupVolumeInputSubMenuMenuItem();
        _volume._volumeMenu.addMenuItem(this._itemInput, 3);
        _volume._volumeMenu._input._maybeShowInput = function() { this._updateVisibility(); }
        _volume._volumeMenu._input._shouldBeVisible = function() { return this._stream !== null; }
        this._updateVisibility();
        this._streamUpdatedId = _volume._volumeMenu._input.connect('stream-updated', Lang.bind(this, this._onChangedIcon));
        this._iconChangedId = _volume._volumeMenu.connect('icon-changed', Lang.bind(this, this._onChangedIcon));
        this._onChangedIcon();
    },

    _getIconName: function() {
        if (!_volume._volumeMenu._input._stream) return null;
        if (_volume._volumeMenu._input._stream.is_muted || _volume._volumeMenu._input._stream.volume <= 0) {
            return 'microphone-sensitivity-muted-symbolic';
        } else {
            let n = Math.floor(3 * _volume._volumeMenu._input._stream.volume / _volume._control.get_vol_max_norm()) + 1;
            if (n < 2) return 'microphone-sensitivity-low-symbolic';
            if (n >= 3) return 'microphone-sensitivity-high-symbolic';
            return 'microphone-sensitivity-medium-symbolic';
        }
    },

    _onChangedIcon: function() {
        let iconName = this._getIconName();
        if (iconName !== null) {
            this._secondaryIndicator.icon_name = iconName;
            this._secondaryIndicator.show();
        } else {
            this._secondaryIndicator.hide();
        }
        _volume.indicators.visible = _volume.indicators.visible || this._secondaryIndicator.visible;
    },

    _updateVisibility: function() {
        _volume._volumeMenu._input._maybeShowInput.call(_volume._volumeMenu._input);
    },

    destroy: function() {
        _volume._volumeMenu.disconnect(this._iconChangedId);
        _volume._volumeMenu._input.disconnect(this._streamUpdatedId);
        _volume._volumeMenu._input._shouldBeVisible = StatusVolume.InputStreamSlider.prototype._shouldBeVisible;
        _volume._volumeMenu._input._maybeShowInput = StatusVolume.InputStreamSlider.prototype._maybeShowInput;
        this._updateVisibility();
        this._itemInput.destroy();
        this._itemOutput.destroy();
        this._secondaryIndicator.destroy();
    }
});

function get() {
    return new VolumeIndicator();
}
