/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Copyright (c) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;

const Clutter = imports.gi.Clutter;
const Pango = imports.gi.Pango;
const St = imports.gi.St;

const ModalDialog = imports.ui.modalDialog;

const ExtensionUtils = imports.misc.extensionUtils;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const _ = Convenience.getTranslations();

const DialogClearRecentDocumentsContent = {
    subject: _("Clear list of recent used documents?"),
    description: _("If you clear the list of recent used documents, you clear the following:\n\u2022 All items from the Places \u2192 Recent used documents menu item\n\u2022 All items from the recent used documents list in all applications"),
    buttons: [
        { signal: 'open-recent-documents', label: _("Show documents") },
        { signal: 'clear-recent-documents', label: _("Clear list") }
    ],
    iconName: 'document-open-recent-symbolic'
};

const DialogEjectVolumeContent = {
    subject: _("Eject the selected volume?"),
    description: function(formatValue) {
        return _("Volume: '%s'").format(formatValue);
    },
    buttons: [
        { signal: 'eject-volume', label: _("Eject") }
    ],
    iconName: 'media-eject-symbolic'
};

const DialogHibernateContent = {
    subject: _("Hibernate"),
    description: _("Do you really want to hibernate the system?"),
    buttons: [
        { signal: 'hibernate', label: _("Hibernate") }
    ],
    iconName: 'document-save-symbolic'
};

const DialogHibernateHybridSleepContent = {
    subject: _("Hibernate or Hybrid-Sleep"),
    description: _("Do you really want to hibernate or hybrid sleep the system?"),
    buttons: [
        { signal: 'hybrid-sleep', label: _("Hybrid-Sleep") },
        { signal: 'hibernate', label: _("Hibernate") }
    ],
    iconName: 'document-save-symbolic'
};

const DialogHybridSleepContent = {
    subject: _("Hybrid-Sleep"),
    description: _("Do you really want to hybrid sleep the system?"),
    buttons: [
        { signal: 'hybrid-sleep', label: _("Hybrid-Sleep") }
    ],
    iconName: 'document-save-as-symbolic'
};

const DialogRemoveRecentDocumentContent = {
    subject: _("Remove the selected document?"),
    description: function(formatValue) {
        return _("Are you sure you want to remove '%s' from the list of recent used documents?").format(formatValue);
    },
    buttons: [
        { signal: 'remove-recent-document', label: _("Remove") }
    ],
    iconName: 'media-eject-symbolic'
};

const DialogTrashEmptyContent = {
    subject: _("Empty all items from Trash?"),
    description: _("All items in the Trash will be permanently deleted."),
    buttons: [
        { signal: 'trash-open', label: _("Open Trash") },
        { signal: 'trash-empty', label: _("Empty the Trash") }
    ],
    iconName: 'user-trash-symbolic'
};

const DialogTrashEntryDeleteContent = {
    subject: _("If you delete an object, it will be permanently lost."),
    description: function(formatValue) {
        return _("Are you sure you want to permanently delete '%s' from the trash?").format(formatValue);
    },
    buttons: [
        { signal: 'trash-entry-delete', label: _("Delete Permanently") }
    ],
    iconName: 'user-trash-symbolic'
};

const DialogContent = {
    'ClearRecentDocuments': DialogClearRecentDocumentsContent,
    'EjectVolume': DialogEjectVolumeContent,
    'Hibernate': DialogHibernateContent,
    'HibernateHybridSleep': DialogHibernateHybridSleepContent,
    'HybridSleep': DialogHybridSleepContent,
    'RemoveRecentDocument': DialogRemoveRecentDocumentContent,
    'TrashEmpty': DialogTrashEmptyContent,
    'TrashEntryDelete': DialogTrashEntryDeleteContent
}

const MODAL_DIALOG_ICON_SIZE = 48;

const ModalUserDialog = new Lang.Class({
    Name: 'ModalUserDialog',
    Extends: ModalDialog.ModalDialog,

    _init: function(content, value) {
        this.parent({ styleClass: 'end-session-dialog' });
        let dialogContent = DialogContent[content];
        let mainContentLayout = new St.BoxLayout({ vertical: false });
        let iconBin = new St.Bin();
        iconBin.set_child(new St.Icon({ icon_name: dialogContent.iconName, icon_size: MODAL_DIALOG_ICON_SIZE, style_class: 'end-session-dialog-shutdown-icon' }));
        mainContentLayout.add(iconBin, { x_fill: true, y_fill: false, x_align: St.Align.END, y_align: St.Align.START });
        let messageLayout = new St.BoxLayout({ vertical: true, style_class: 'end-session-dialog-layout' });
        let subjectLabel = new St.Label({ text: dialogContent.subject, style_class: 'end-session-dialog-subject' });
        messageLayout.add(subjectLabel, { x_fill: false, y_fill: false, x_align: St.Align.START, y_align: St.Align.START });
        let descriptionLabel = new St.Label({ text: value ? dialogContent.description(value) : dialogContent.description, style_class: 'end-session-dialog-description' });
        descriptionLabel.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
        descriptionLabel.clutter_text.line_wrap = true;
        messageLayout.add(descriptionLabel, { y_fill: true, y_align: St.Align.START });
        mainContentLayout.add(messageLayout, { y_align: St.Align.START });
        this.contentLayout.add(mainContentLayout, { x_fill: true, y_fill: false });
        let buttons = [
            { action: Lang.bind(this, this.close), label: _("Cancel"), key: Clutter.Escape }
        ];
        dialogContent.buttons.forEach(function(button) {
            buttons.push({ action: Lang.bind(this, function() {
                let signalId = this.connect('closed', Lang.bind(this, function() {
                    this.disconnect(signalId);
                    this.emit(button.signal);
                }));
                this.close();
            }), label: button.label });
        }, this);
        this.setButtons(buttons);
    }
});
