/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

  Copyright (C) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.

*/

const Lang = imports.lang;

const Clutter = imports.gi.Clutter;
const St = imports.gi.St;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils = imports.misc.extensionUtils;
const Util = imports.misc.util;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;
const Sensors = Me.imports.sensors;

const _ = Convenience.getTranslations();

const KEY_REFRESH_INTERVAL = 'refresh-interval';
const KEY_SHOW_FAHRENHEIT = 'show-fahrenheit';
const KEY_SHOW_TEMPERATURE = 'show-temperature';

const TEMPERATURE_AVERAGE = 'average';
const TEMPERATURE_DISABLED = '';
const TEMPERATURE_HIGHEST = 'highest';

function getTemperatureAsString(temperature) {
    if (!temperature || temperature === 0) return 'N/A';
    if (Convenience.Settings.get_boolean(KEY_SHOW_FAHRENHEIT)) {
        return Math.round(temperature * 1.8 + 32).toFixed(0)+' \u00b0F';
    } else {
        return Math.round(temperature).toFixed(1)+' \u00b0C';
    }
}

const PopupTemperatureMenuItem = new Lang.Class({
    Name: 'PopupTemperatureMenuItem',
    Extends: PopupMenu.PopupBaseMenuItem,

    _init: function(text, iconName, params) {
        this.parent(params);
        if (iconName) {
            this._icon = new St.Icon({ icon_name: iconName, style_class: 'popup-menu-icon' });
            this.actor.add_child(this._icon);
        }
        this._label = new St.Label({ text: text, x_expand: true, y_align: Clutter.ActorAlign.CENTER });
        this.actor.add_child(this._label);
        this.actor.label_actor = this._label;
        this._settingsChangedId = [
            Convenience.Settings.connect('changed::'+KEY_SHOW_FAHRENHEIT, Lang.bind(this, this._onChangedShowFahrenheit)),
            Convenience.Settings.connect('changed::'+KEY_SHOW_TEMPERATURE, Lang.bind(this, this._onChangedShowTemperature))
        ];
    },

    _onChangedShowFahrenheit: function() {
        if (!this._statusLabel) return;
        this._statusLabel.set_text(getTemperatureAsString(this._status));
    },

    _onChangedShowTemperature: function() {
        if (!this._id) return;
        let showTemperature = Convenience.Settings.get_strv(KEY_SHOW_TEMPERATURE);
        this._selected = showTemperature.indexOf(this._id) >= 0;
        this.setOrnament(this._selected ? PopupMenu.Ornament.CHECK : PopupMenu.Ornament.NONE);
    },

    activate: function(event) {
        if (this._id) Convenience.Settings.set_strv(KEY_SHOW_TEMPERATURE, [ this._selected ? TEMPERATURE_DISABLED : this._id ]);
        this.parent(event);
    },

    destroy: function() {
        this._settingsChangedId.forEach(function(id) {
            Convenience.Settings.disconnect(id);
        });
        this.parent();
    },

    setId: function(id) {
        this._id = id;
        this._onChangedShowTemperature();
    },

    setStatus: function(status) {
        if (!this._statusLabel) {
            this._statusLabel = new St.Label({ y_align: Clutter.ActorAlign.CENTER });
            this.actor.add_child(this._statusLabel);
        }
        this._status = status;
        this._onChangedShowFahrenheit();
    }
});

const PopupTemperatureSubMenuMenuItem = new Lang.Class({
    Name: 'PopupTemperatureSubMenuMenuItem',
    Extends: PopupMenu.PopupSubMenuMenuItem,

    _init: function(text, iconName) {
        this.parent(text, true);
        this.icon.icon_name = iconName;
        this._statusLabel = new St.Label({ y_expand: true, y_align: Clutter.ActorAlign.CENTER });
        let children = this.actor.get_n_children();
        this.actor.insert_child_at_index(this._statusLabel, children - 1);
        this._settingsChangedId = Convenience.Settings.connect('changed::'+KEY_SHOW_FAHRENHEIT, Lang.bind(this, this._onChangedShowFahrenheit));
    },

    _onChangedShowFahrenheit: function() {
        this._statusLabel.set_text(getTemperatureAsString(this._status));
    },

    addMenuItem: function(text, status) {
        let item = new PopupTemperatureMenuItem(text, null, { reactive: false });
        item.setStatus(status);
        this.menu.addMenuItem(item);
    },

    destroy: function() {
        Convenience.Settings.disconnect(this._settingsChangedId);
        this.parent();
    },

    setStatus: function(status) {
        this._status = status;
        this._onChangedShowFahrenheit();
    }
});

const PopupSensorSubMenuMenuItem = new Lang.Class({
    Name: 'PopupSensorSubMenuMenuItem',
    Extends: PopupTemperatureSubMenuMenuItem,

    _init: function(sensor, text, iconName) {
        this.parent(text, iconName)
        this._sensor = sensor;
        this._sensorChangedId = this._sensor.connect('updated::sensor', Lang.bind(this, this._onUpdatedSensor));
        this._onUpdatedSensor();
    },

    _onUpdatedSensor: function() {
        this.menu.removeAll();
        this._sensor.children.forEach(function(child) {
            this.addMenuItem(child.label, child.current);
        }, this);
        this.setStatus(this._sensor.highest);
        this.actor.visible = this.menu.numMenuItems > 0;
    },

    destroy: function() {
        this._sensor.disconnect(this._sensorChangedId);
        this.parent();
    }
});

const PopupSensorMenuSection = new Lang.Class({
    Name: 'PopupSensorMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function(sensor) {
        this.parent();
        this._sensor = sensor;
        this._sensorChangedId = this._sensor.connect('updated::sensor', Lang.bind(this, this._onUpdatedSensor));
        this._onUpdatedSensor();
    },

    _onUpdatedSensor: function() {
        this.removeAll();
        this._sensor.children.forEach(function(child) {
            let item;
            if (child.maximum || child.critical) {
                item = new PopupTemperatureSubMenuMenuItem(child.label, child.warning ? 'dialog-warning' : 'utilities-system-monitor');
                item.setStatus(child.current);
                item.addMenuItem(_("Current temperature"), child.current);
                if (child.maximum) item.addMenuItem(_("Maximum temperature"), child.maximum);
                if (child.critical) item.addMenuItem(_("Critical temperature"), child.critical);
            } else {
                item = new PopupTemperatureMenuItem(child.label, 'utilities-system-monitor', { reactive: false });
                item.setStatus(child.current);
            }
            this.addMenuItem(item);
        }, this);
        this.actor.visible = this.numMenuItems > 0;
    },

    destroy: function() {
        this._sensor.disconnect(this._sensorChangedId);
        this.parent();
    }
});

const TemperatureMenuButton = new Lang.Class({
    Name: 'TemperatureMenuButton',
    Extends: PanelMenu.Button,

    _init: function() {
        this.parent(0.5, _("Temperature"), false);
        this.menu.actor.add_style_class_name('dschaerf-menu');
        this._hbox = new St.BoxLayout({ style_class: 'panel-status-indicators-box' });
        this._icon = new St.Icon({ icon_name: 'view-refresh-symbolic', style_class: 'system-status-icon'});
        this._hbox.add_child(this._icon);
        this._label = new St.Label({ text: '...', y_align: Clutter.ActorAlign.CENTER }),
        this._hbox.add_child(this._label);
        this._hbox.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
        this.actor.add_child(this._hbox);
        this._sensorManager = new Sensors.SensorManager();
        this._itemNotAvailable = new PopupTemperatureMenuItem(_("No supported temperature sensors available"), 'dialog-warning', { reactive: false, can_focus: false });
        this.menu.addMenuItem(this._itemNotAvailable);
        this._itemSensorHWM = new PopupSensorMenuSection(this._sensorManager.get('hwm'));
        this.menu.addMenuItem(this._itemSensorHWM);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemSensorVGA = new PopupSensorSubMenuMenuItem(this._sensorManager.get('vga'), _("Graphics Cards"), 'video-display');
        this.menu.addMenuItem(this._itemSensorVGA);
        this._itemSensorHDD = new PopupSensorSubMenuMenuItem(this._sensorManager.get('hdd'), _("Hard disk drives"), 'drive-harddisk');
        this.menu.addMenuItem(this._itemSensorHDD);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemAverage = new PopupTemperatureMenuItem(_("Average temperature"), 'utilities-system-monitor');
        this._itemAverage.setId('average');
        this.menu.addMenuItem(this._itemAverage);
        this._itemHighest = new PopupTemperatureMenuItem(_("Highest temperature"), 'utilities-system-monitor');
        this._itemHighest.setId('highest');
        this.menu.addMenuItem(this._itemHighest);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._itemActions = new PopupMenu.PopupBaseMenuItem({ reactive: false, can_focus: false });
        this._actionSettings = this._createActionButton('preferences-other-symbolic', _("Settings"));
        this._actionSettings.connect('clicked', Lang.bind(this, this._onClickedSettings));
        this._itemActions.actor.add(this._actionSettings, { expand: true, x_fill: false });
        this._actionRefresh = this._createActionButton('view-refresh-symbolic', _("Settings"));
        this._actionRefresh.connect('clicked', Lang.bind(this, this._onClickedRefresh));
        this._itemActions.actor.add(this._actionRefresh, { expand: true, x_fill: false });
        this.menu.addMenuItem(this._itemActions);
        this._sensorManagerChangedId = this._sensorManager.connect('updated::sensors', Lang.bind(this, this._onUpdatedSensors));
        this._settingsChangedId = [
            Convenience.Settings.connect('changed::'+KEY_REFRESH_INTERVAL, Lang.bind(this, this._onChangedRefreshInterval)),
            Convenience.Settings.connect('changed::'+KEY_SHOW_FAHRENHEIT, Lang.bind(this, this._updateLabel)),
            Convenience.Settings.connect('changed::'+KEY_SHOW_TEMPERATURE, Lang.bind(this, this._updateLabel))
        ];
        this._onChangedRefreshInterval();
    },

    _createActionButton: function(iconName, accessibleName) {
        let icon = new St.Button({ reactive: true, can_focus: true, track_hover: true, accessible_name: accessibleName, style_class: 'system-menu-action' });
        icon.child = new St.Icon({ icon_name: iconName });
        return icon;
    },

    _onChangedRefreshInterval: function() {
        let refreshInterval = Convenience.Settings.get_string(KEY_REFRESH_INTERVAL);
        this._sensorManager.setRefreshInterval(parseInt(refreshInterval));
    },

    _onClickedRefresh: function() {
        this._onChangedRefreshInterval();
    },

    _onClickedSettings: function() {
        this.menu.itemActivated();
        Util.spawnCommandLine('gnome-shell-extension-prefs '+Me.metadata['uuid']);
        Main.overview.hide();
    },

    _onOpenStateChanged: function(menu, open) {
        if (open) {
            let available = this._sensorManager.available;
            this._itemAverage.actor.visible = available;
            this._itemHighest.actor.visible = available;
            this._itemActions.actor.visible = available;
            this._itemNotAvailable.actor.visible = !available;
        }
        this.parent(menu, open);
    },

    _onUpdatedSensors: function() {
        this._itemAverage.setStatus(this._sensorManager.average);
        this._itemHighest.setStatus(this._sensorManager.highest);
        this._updateIcon();
        this._updateLabel();
    },

    _updateIcon: function() {
        let iconName = 'utilities-system-monitor-symbolic';
        if (!this._sensorManager.available || this._sensorManager.warning) iconName = 'dialog-warning-symbolic';
        this._icon.icon_name = iconName;
    },

    _updateLabel: function() {
        let showTemperature = Convenience.Settings.get_strv(KEY_SHOW_TEMPERATURE)[0];
        if (showTemperature !== TEMPERATURE_DISABLED) {
            let temperature = this._sensorManager[showTemperature];
            this._label.set_text(getTemperatureAsString(temperature));
            this._label.show();
        } else {
            this._label.hide();
        }
    },

    destroy: function() {
        this._sensorManager.disconnect(this._sensorManagerChangedId);
        this._settingsChangedId.forEach(function(id) {
            Convenience.Settings.disconnect(id);
        });
        this._sensorManager.destroy();
        this.parent();
    }
});

function init() {
    Convenience.initTranslations();
}

let _extension = null;

function enable() {
    if (_extension === null) _extension = Main.panel.addToStatusArea('dschaerfTemperatureMenu', new TemperatureMenuButton(), 1, 'center');
}

function disable() {
    if (_extension !== null) {
        _extension.destroy();
        _extension = null;
    }
}
