/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
  Copyright (c) 2012-2021, Dieter Schärf <ds@dschaerf.de>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const GObject = imports.gi.GObject;

const ExtensionUtils = imports.misc.extensionUtils;

const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;

const _ = Convenience.getTranslations();

const SETTINGS_GENERALLY = [
    {
        type: 'combo',
        key: 'refresh-interval',
        label: _("Update interval in seconds"),
        tooltip: _("Sets the update interval for the temperature query of the sensors in seconds."),
        values: {
            '10': '10',
            '15': '15',
            '30': '30',
            '60': '60',
            '120': '120',
            '300': '300',
            '600': '600'
        }
    },
    {
        type: 'switcher',
        key: 'show-fahrenheit',
        label: _("Show temperature in Fahrenheit"),
        tooltip: _("When this option is activated, Fahrenheit is used as the temperature unit for the display instead of Celsius.")
    }
];

const TemperaturePrefsWidget = new GObject.Class({
    Name: 'Temperature.Prefs.Widget',
    GTypeName: 'TemperaturePrefsWidget',
    Extends: Gtk.Notebook,

    _init: function(params) {
        this.parent(params);
        this._createPrefsWidget(SETTINGS_GENERALLY, _("Generally"));
    },

    _createPrefsWidget: function(prefs, text) {
        let box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin: 15, spacing: 15 });
        prefs.forEach(function(pref) {
            let widget = this._createWidget(pref);
            if (widget) box.add(widget);
        }, this);
        this.append_page(box, new Gtk.Label({ label: text }));
    },

    _createWidget: function(pref) {
        let settings = Convenience.getSettings(pref.schema ? pref.schema : null);
        let widget = null;
        widget = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
        widget.pack_start(new Gtk.Label({ label: _(pref.label), xalign: 0 }), true, true, 0);
        let prefAction;
        switch (pref.type) {
        case 'combo':
            prefAction = new Gtk.ComboBoxText();
            for (let value in pref.values) {
                prefAction.append(value, _(pref.values[value]));
            }
            prefAction.connect('changed', function() {
                settings.set_string(pref.key, prefAction.get_active_id());
            });
            prefAction.set_active_id(settings.get_string(pref.key));
            settings.connect('changed::'+pref.key, function() {
                prefAction.set_active_id(settings.get_string(pref.key));
            });
            break;
        case 'switcher':
            prefAction = new Gtk.Switch({ active: settings.get_boolean(pref.key) });
            prefAction.connect('notify::active', function() {
                settings.set_boolean(pref.key, prefAction.get_active());
            });
            settings.connect('changed::'+pref.key, function() {
                prefAction.set_active(settings.get_boolean(pref.key));
            });
            break;
        default: // Do nothing
        }
        prefAction.height_request = 24;
        if (pref.type !== 'switcher') prefAction.width_request = 250;
        if (pref.tooltip) prefAction.set_tooltip_text(_(pref.tooltip));
        widget.pack_end(prefAction, false, false, 0);
        return widget;
    }
});

function init(metadata) {
    Convenience.initTranslations();
}

function buildPrefsWidget() {
    let widget = new TemperaturePrefsWidget({ scrollable: true, margin: 15, width_request: 720, height_request: 480 });
    widget.show_all();
    return widget;
}
